#pragma once

// TODO: conflict glew / glad -- igl viewer uses glad, we use glew... -> don't use igl viewer

#include <iostream>
#include <sstream>
#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <glm/glm.hpp>
#include <renderdoc_app.h>

#define FATAL_ERROR(err) { \
	std::cout << "FATAL ERROR: " << err << std::endl; \
	system("PAUSE"); \
	exit(-1); }

#define PRINT_ERROR(err) { \
	std::cout << "ERROR: " << err << std::endl; \
	}

#define PRINT_OUT(x) { \
	std::cout << x << std::endl; \
	}

#define MYASSERT(expr,msg) if (!(expr)) FATAL_ERROR("Assertion failed: " << msg)


#define DELETE_VECTOR_CONTENTS(vect,type) { while (!(vect).empty()) { type elem = (vect).back(); (vect).pop_back(); delete elem; } }
#define DELETE_AND_NULL(x) { if (x) {delete x; x = nullptr; }}


// debug labeling for RenderDoc/NSight
//#define ENABLE_LABELED_SECTIONS _DEBUG
#define ENABLE_LABELED_SECTIONS 1

#ifdef USE_NVIDIA_TOOLS_EXTENSION
#define LABELED_SECTIONS_USE_NVXT 1
#endif

#if ENABLE_LABELED_SECTIONS
#ifdef LABELED_SECTIONS_USE_NVXT
#define LABELED_SECTION_BEGIN(x) nvtxRangePushA((x))
#define LABELED_SECTION_END nvtxRangePop()
#else
#define LABELED_SECTION_BEGIN(x) glPushDebugGroup(GL_DEBUG_SOURCE_APPLICATION, 1, -1, (x))
#define LABELED_SECTION_END glPopDebugGroup()
#endif
#else
#define LABELED_SECTION_BEGIN(x)
#define LABELED_SECTION_END
#endif

#define USE_RENDERDOC_API _DEBUG

extern RENDERDOC_API_1_1_2* rdoc_api;


void pushFramebuffer();
void popFramebuffer();

void APIENTRY openGL_debugCallback(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, const GLchar* message, const GLvoid* userParam);

// -- globals --

// Assets directory
extern std::string gAssetsDir;

void initGlobals();


