#version 430 core

// see contour.geom for details
// this variant only renders the contour edges as lines

layout (triangles) in;
layout (line_strip, max_vertices = 2) out;

in VertexData {
	vec3 positionVS;
	vec3 normalVS;
	vec2 UV;
} gs_in[3];

out GeoOut {
	vec3 positionVS;
	vec3 normalVS;
	float paramU; // TODO!!
	float paramQ;
	float isContour;
} gs_out;

uniform mat4 matP;

vec3 alpha; // alpha[i] contains dot(normal[i], camDir[i])

float contourFactor(int i1, int i2) {
	return -alpha[i2] / (alpha[i1] - alpha[i2]);
}

vec3 toContour(vec3 vA, vec3 vB, float mixFactor) {
	return mix(vB, vA, mixFactor);
}

void outputVertex(vec3 posVS, vec3 nrmVS, float u, float q, float isContour) {
	gl_Position = matP * vec4(posVS, 1);
	gs_out.positionVS  = posVS;
	gs_out.normalVS    = nrmVS;
	gs_out.paramU      = u;
	gs_out.paramQ      = q;
	gs_out.isContour   = isContour;
	EmitVertex();
}

void makeLine(int i1, int i2, int i3) {
	// edges i1-i3 and i2-i3 intersect contour (i1 and i2 are on the "same side", i.e. both either visible or invisible)
	float f13 = contourFactor(i1, i3);
	vec3  p13 = toContour(gs_in[i1].positionVS, gs_in[i3].positionVS, f13);
	float f23 = contourFactor(i2, i3);
	vec3  p23 = toContour(gs_in[i2].positionVS, gs_in[i3].positionVS, f23);

	const float q = 0;

	outputVertex(p13, vec3(0), 0, q, 1);
	outputVertex(p23, vec3(0), 0, q, 1);
	EndPrimitive();
}

void main() {

	// dot vertex normal with to-eye-vector
	alpha[0] = dot(gs_in[0].normalVS, -gs_in[0].positionVS);
	alpha[1] = dot(gs_in[1].normalVS, -gs_in[1].positionVS);
	alpha[2] = dot(gs_in[2].normalVS, -gs_in[2].positionVS);

	bvec3 vis = greaterThanEqual(alpha, vec3(0));

	if (!all(vis) && !all(!vis)) {
		if      (vis.x == vis.y) makeLine(0,1,2);
		else if (vis.y == vis.z) makeLine(1,2,0);
		else if (vis.z == vis.x) makeLine(2,0,1);
	}
}
