#version 430 core

layout (location=0) out vec4 fragColor;


uniform ivec2 screenSize;
uniform int stepSize;
uniform sampler2D texInput;
uniform bool depthAware = true;
uniform bool makeVtexture = false;

// RGBA layout:
// closest seed cell is stored in .xy (uv coords). Unprocessed cells have -1,-1
// .z: screen depth
// .w: is-contour flag

void main() {

	float minSqDist = dot(screenSize, screenSize) * 100.0;
	vec2 minSeed = vec2(-1,-1);

	ivec2 myCell = ivec2(floor(gl_FragCoord.xy));
	vec4  myVal  = texelFetch(texInput, myCell, 0);

	// sample the neighbourhood
	for (int dx = -1; dx < 2; dx++) {
		for (int dy = -1; dy < 2; dy++) {
			// texture sampler is set up to clamp to edge, so we can safely ignore boundaries - ok for texture(), however undefined behaviour for texelFetch()!
			// so clamp manually
			ivec2 loc = clamp(myCell + ivec2(dx,dy) * stepSize, ivec2(0), screenSize - 1);
			vec4 val = texelFetch(texInput, loc, 0);

			// if this texel has any information...
			if (val.x >= 0 && val.y >= 0) {
				// ...and the referenced seed cell is not closer to the viewer than the pixel
				float seedDepth = texelFetch(texInput, ivec2(floor(val.xy)), 0).z;
				if (seedDepth >= myVal.z || !depthAware) {
					// ...calc distance to the referenced seed cell
					vec2 delta = val.xy - vec2(myCell);
					float sqDist = dot(delta, delta);
					// ...and remember the closest seed
					if (sqDist < minSqDist) {
						minSqDist = sqDist;
						minSeed = val.xy;
					}
				}
			}
		}
	}


	if (makeVtexture) {
		float d = sqrt(minSqDist);
		fragColor = vec4(d,0,0,myVal.w); // keep is-contour-flag
	} else {
		fragColor = vec4(minSeed, myVal.zw);
	}
}
