@page technical Technical documentation

Short description of the technical implementation details.

# Libraries

External libraries used are
- \ref refEIGEN "Eigen" for matrix algebra
- \ref refLIBIGL "libigl" for mesh geometry computations and .obj/.off file parsing
- \ref refIMGUI "Dear ImGui" for the graphical user interface
- \ref refMINI "mini" for ini-file management (loading/saving settings)
- \ref refSTB "stb_image" for loading textures
- \ref refPFD "portable-file-dialogs" for file open/save dialogues


# Endpoints and U-parametrisation

**These steps only need to be computed once after loading the mesh.**

After a mesh is loaded from a file, a skeletonised version is calculated by iterative contraction.
This is done by an implementation of \ref refAu08 "Skeleton Extraction by Mesh Contraction".

On this skeletonised mesh, endpoints are detected by placing a small sphere at each vertex and
checking if all other vertices contained in that sphere are located in the same half-plane.
If that is the case, the examined vertex is considered an endpoint.

From all detected endpoints, the one where the mesh curvature is minimal is designated as root-point.

<img src="endpoints.png" align="left" width="50%">
<div style="clear: both"></div>

The U-coordinates are then obtained as described in \ref refLi18 "Parameterization and Feature Extraction for the Visualization of Tree-like Structures":
Minimal curvature is computed per triangle; as the sign of the curvature direction is ambiguous, the geodesic distance from the root point is
computed per vertex, and the per-triangle gradient of the distance is used to resolve the curvature direction.
The resulting vector field is smoothed, and the U-parametrisation is calculated by solving a Poisson equation, so that U is the gradient of the smoothed vector field.

# Deferred rendering pipeline and V-parametrisation

**These steps are repeated for every frame.**

The rendering pipeline commences in multiple steps.

First, the mesh geometry is rendered to G-Buffers:
A geometry shader computes the contour lines of the projected mesh, and adds additional geometry for the contours.
Positions and normals (as well as flags differentiating between contour, mesh and background) are written to G-Buffers.

In the next step, the \ref refRo06 "Jump-Flood Algorithm" (JFA) is used to obtain the distance from the contour for each pixel covered by the mesh.
This is an iterative procedure, employing a ping-pong buffer to use the results of one iteration as input of the next one.
The actual number of iterations depends on the current window resolution.

The result of JFA represents the V-parametrisation (stored in a per-pixel V-Buffer) and looks similar to this:

<img src="jfa.jpg" align="left" width="25%">
<div style="clear: both"></div>

Now that both U- and V-parametrisations are available, the main scene is rendered.
This is accomplished by drawing a full-screen-quad and taking the relevant positions, normals, geometry-type-information and V-parametrisation from the generated G- and V-Buffers.
The selected visualisation techniques are applied in the fragment shader of this step.

Finally, endpoints are rendered:
For each endpoint a semi-translucent sphere is rendered and alpha-blended with the final scene.








